#!/usr/bin/env python

import json
import time
import sys
import os
from collections import OrderedDict as dict


def json_to_js(json_path, js_path):
    """Convert `commands.json` to `commands.js`"""
    keys = []
    with open(json_path) as fp:
        _json = json.load(fp)
        for k in _json.keys():
            keys.append(k.encode('utf-8'))
    with open(js_path, "w") as fp:
        generate_time(fp)
        fp.write("module.exports = [\n")
        for k in sorted(keys):
            fp.write('\t"%s",\n' % k.lower())
        fp.write("]")


def json_to_go_array(json_path, go_path):
    g_fp = open(go_path, "w")
    with open(json_path) as fp:
        _json = json.load(fp)
        generate_time(g_fp)
        g_fp.write("package main\n\nvar helpCommands = [][]string{\n")
        _json_sorted = dict(sorted(_json.items(), key=lambda x: x[0]))
        for k, v in _json_sorted.iteritems():
            g_fp.write('\t{"%s", "%s", "%s"},\n' % (k, v["arguments"], v["group"]))
        g_fp.write("}\n")
    g_fp.close()


def generate_time(fp):
    fp.write("//This file was generated by .tools/generate_commands.py on %s \n" %
             time.strftime('%a %b %d %Y %H:%M:%S %z'))


if __name__ == "__main__":
    usage = """
    Usage: python %s src_path dst_path"

    1. for Node.js client:

        python generate.py /path/to/commands.json /path/to/commands.js

    2. for cmd/ledis_cli/const.go
        
        python generate.py /path/to/commands.json /path/to/const.go

    """

    if len(sys.argv) != 3:
        sys.exit(usage % os.path.basename(sys.argv[0]))

    src_path, dst_path = sys.argv[1:]
    dst_path_base = os.path.basename(dst_path)

    if dst_path_base.endswith(".js"):
        json_to_js(src_path, dst_path)

    elif dst_path_base.startswith("const.go"):
        json_to_go_array(src_path, dst_path)

    else:
        print "Not support arguments"



#coding: utf-8

import random, string

import redis
import ledis

from redis_import import copy, scan, set_ttl

rds = redis.Redis()
lds = ledis.Ledis(port=6380)


def random_word(words, length):
    return ''.join(random.choice(words) for i in range(length))


def get_words():
    word_file = "/usr/share/dict/words"
    words = open(word_file).read().splitlines()
    return words[:1000]


def get_mapping(words, length=1000):
    d = {}
    for word in words:
        d[word] = random.randint(1, length)
    return d


def random_string(client, words, length=1000):
    d = get_mapping(words, length)
    client.mset(d)


def random_hash(client, words, length=1000):
    d = get_mapping(words, length)
    client.hmset("hashName", d)


def random_list(client, words, length=1000):
    client.lpush("listName", *words)


def random_zset(client, words, length=1000):
    d = get_mapping(words, length)
    client.zadd("zsetName", **d)


def test():
    words = get_words()
    print "Flush all redis data before insert new."
    rds.flushall()

    random_string(rds, words)
    print "random_string done"

    random_hash(rds, words)
    print "random_hash done"
    
    random_list(rds, words)
    print "random_list done"

    random_zset(rds, words)
    print "random_zset done"


    lds.lclear("listName")
    lds.hclear("hashName")
    lds.zclear("zsetName")
    copy(rds, lds, convert=True)

    # for all keys
    keys = scan(rds, 1000)
    for key in keys:
        if rds.type(key) == "string" and not lds.exists(key):
            print key
            print "String data not consistent"

    # for list
    l1 = rds.lrange("listName", 0, -1)
    l2 = lds.lrange("listName", 0, -1)
    assert l1 == l2
   
    #for hash
    for key in keys:
        if rds.type(key) == "hash":
            assert rds.hgetall(key) == lds.hgetall(key)
            assert sorted(rds.hkeys(key)) == sorted(lds.hkeys(key))
            assert sorted(rds.hvals(key)) == sorted(lds.hvals(key))

    # for zset
    z1 = rds.zrange("zsetName", 0, -1, withscores=True)
    z2 = lds.zrange("zsetName", 0, -1, withscores=True)
    assert z1 == z2


def ledis_ttl(ledis_client, key, k_type):
    ttls = {
        "string": lds.ttl,
        "list": lds.lttl,
        "hash": lds.httl,
        "zset": lds.zttl,
    }
    return ttls[k_type](key)


def test_ttl():
    keys, total = scan(rds, 1000)
    for key in keys:
        k_type = rds.type(key)
        rds.expire(key, (60 * 60 * 24))
        set_ttl(rds, lds, key, k_type)
        if rds.ttl(key):
            assert ledis_ttl(lds, key, k_type) > 0

if __name__ == "__main__":
    test()
    test_ttl()
    print "Test passed."



#!/usr/bin/env python
# coding: utf-8

# refer: https://github.com/ideawu/ssdb/blob/master/tools/redis-import.php

# Notice: for zset, float score will be converted to integer.

import sys
import os
from collections import OrderedDict as od

import redis
import ledis

total = 0
entries = 0


def scan_available(redis_client):
    """"Scan Command is available since redis-server 2.8.0"""

    if "scan" in dir(redis_client):
        info = redis_client.info()
        server_version = info["redis_version"]
        version_list = server_version.split(".")
        if len(version_list) > 2:
            n = int(version_list[0]) * 10 + int(version_list[1])
            if n >= 28:
                return True
    return False


def set_ttl(redis_client, ledis_client, key, k_type):
    k_types = {
        "string": ledis_client.expire,
        "list": ledis_client.lexpire,
        "hash": ledis_client.hexpire,
        "set": ledis_client.zexpire,
        "zset": ledis_client.zexpire
    }
    timeout = redis_client.ttl(key)
    if timeout > 0:
        k_types[k_type](key, timeout)


def copy_key(redis_client, ledis_client, key, convert=False):
    global entries
    k_type = redis_client.type(key)
    if k_type == "string":
        value = redis_client.get(key)
        ledis_client.set(key, value)
        set_ttl(redis_client, ledis_client, key, k_type)
        entries += 1

    elif k_type == "list":
        _list = redis_client.lrange(key, 0, -1)
        for value in _list:
            ledis_client.rpush(key, value)
        set_ttl(redis_client, ledis_client, key, k_type)
        entries += 1

    elif k_type == "hash":
        mapping = od(redis_client.hgetall(key))
        ledis_client.hmset(key, mapping)
        set_ttl(redis_client, ledis_client, key, k_type)
        entries += 1

    elif k_type == "zset":
        out = redis_client.zrange(key, 0, -1, withscores=True)
        pieces = od()
        for i in od(out).iteritems():
            pieces[i[0]] = int(i[1])
        ledis_client.zadd(key, **pieces)
        set_ttl(redis_client, ledis_client, key, k_type)
        entries += 1

    else:
        print "KEY %s of TYPE %s is not supported by LedisDB." % (key, k_type)


def copy_keys(redis_client, ledis_client, keys, convert=False):
    for key in keys:
        copy_key(redis_client, ledis_client, key, convert=convert)


def scan(redis_client, count=1000):
    keys = []
    total = redis_client.dbsize()
    if total > 1000:
        print "It may take a while, be patient please."

    first = True
    cursor = 0
    while cursor != 0 or first:
        cursor, data = redis_client.scan(cursor, count=count)
        keys.extend(data)
        first = False
    assert len(keys) == total
    return keys, total


def copy(redis_client, ledis_client, count=1000, convert=False):
    if scan_available(redis_client):
        print "\nTransfer begin ...\n"
        keys, total = scan(redis_client, count=count)
        copy_keys(redis_client, ledis_client, keys, convert=convert)

    else:
        msg = """We do not support Redis version less than 2.8.0.
            Please check both your redis server version and redis-py
            version.
              """
        print msg
        sys.exit()
    print "%d keys, %d entries copied" % (total, entries)


def usage():
    usage = """
        Usage:
        python %s redis_host redis_port redis_db ledis_host ledis_port
        """
    print usage % os.path.basename(sys.argv[0])


def get_prompt(choice):
    yes = set(['yes', 'ye', 'y', ''])
    no = set(['no', 'n'])

    if choice in yes:
        return True
    elif choice in no:
        return False
    else:
        sys.stdout.write("Please respond with 'yes' or 'no'")


def main():
    if len(sys.argv) < 6:
        usage()
        sys.exit()
    convert = False
    if len(sys.argv) >= 6:
        (redis_host, redis_port, redis_db, ledis_host, ledis_port) = sys.argv[1:6]
        if int(redis_db) >= 16:
            print redis_db
            sys.exit("LedisDB only support 16 databases([0-15]")

    choice = raw_input("[y/N]").lower()
    if not get_prompt(choice):
        sys.exit("No proceed")

    redis_c = redis.Redis(host=redis_host, port=int(redis_port), db=int(redis_db))
    ledis_c = ledis.Ledis(host=ledis_host, port=int(ledis_port), db=int(redis_db))
    try:
        redis_c.ping()
    except redis.ConnectionError:
        print "Could not connect to Redis Server"
        sys.exit()

    try:
        ledis_c.ping()
    except redis.ConnectionError:
        print "Could not connect to LedisDB Server"
        sys.exit()

    copy(redis_c, ledis_c, convert=convert)
    print "Done\n"


if __name__ == "__main__":
    main()
